from netCDF4 import Dataset

e280 = 1
e400 = 2
ei280 = 3
ei400 = 4
eo280 = 5
eo400 = 6
eoi280 = 7
eoi400 = 8

AllClims = {}

AllClims[e280] = Dataset("E280.nc", "r")
AllClims[e400] = Dataset("E400.nc", "r")
AllClims[ei280] = Dataset("Ei280.nc", "r")
AllClims[ei400] = Dataset("Ei400.nc", "r")
AllClims[eo280] = Dataset("Eo280.nc", "r")
AllClims[eo400] = Dataset("Eo400.nc", "r")
AllClims[eoi280] = Dataset("Eoi280.nc", "r")
AllClims[eoi400] = Dataset("Eoi400.nc", "r")



def factorize_Lunt2012():
    """
    Factorize an anomaly into contributions from each of the different changes to the boundary conditions
    using the methodology of Lunt et al. 2012 and Haywood et al. 2016
    
    Returns:
        1. The total change between pliocene and PI
        2. The factorized contribution from CO2
        3. The factorized contribution from topography
        4. The factorized contribution from ice
    """
    var = "atm/TREFHT"
    dT = AllClims[eoi400][var][:, :] - AllClims[e280][var][:, :]

    dTCO2 = (AllClims[e400][var][:, :] - AllClims[e280][var][:, :] +
             AllClims[eo400][var][:, :] - AllClims[eo280][var][:, :] +
             AllClims[ei400][var][:, :] - AllClims[ei280][var][:, :] +
             AllClims[eoi400][var][:, :] - AllClims[eoi280][var][:, :]) / 4

    dTtopo = (AllClims[eo280][var][:, :] - AllClims[e280][var][:, :] +
              AllClims[eo400][var][:, :] - AllClims[e400][var][:, :] +
              AllClims[eoi280][var][:, :] - AllClims[ei280][var][:, :] +
              AllClims[eoi400][var][:, :] - AllClims[ei400][var][:, :]) / 4

    dTice = (AllClims[ei280][var][:, :] - AllClims[e280][var][:, :] +
             AllClims[ei400][var][:, :] - AllClims[e400][var][:, :] +
             AllClims[eoi280][var][:, :] - AllClims[eo280][var][:, :] +
             AllClims[eoi400][var][:, :] - AllClims[eo400][var][:, :]) / 4

    return dT, dTCO2, dTtopo, dTice


def factorizeComplete():
    """
    Factorize an anomaly into contributions from each of the different changes to the boundary conditions
    using the all-linear and complete methodology of Lunt et al. 2020.
    
    Returns:
        1. The total change between pliocene and PI
        2. The factorized contribution from CO2
        3. The factorized contribution from topography
        4. The factorized contribution from ice
    """

    var = "atm/TREFHT"
    dT = AllClims[eoi400][var][:, :] - AllClims[e280][var][:, :]

    dTCO2 = (2 * (AllClims[e400][var][:, :] - AllClims[e280][var][:, :]) +
             (AllClims[eo400][var][:, :] - AllClims[eo280][var][:, :]) +
             (AllClims[ei400][var][:, :] - AllClims[ei280][var][:, :]) +
             2 * (AllClims[eoi400][var][:, :] - AllClims[eoi280][var][:, :])) / 6

    dTtopo = (2 * (AllClims[eo280][var][:, :] - AllClims[e280][var][:, :]) +
              (AllClims[eo400][var][:, :] - AllClims[e400][var][:, :]) +
              (AllClims[eoi280][var][:, :] - AllClims[ei280][var][:, :]) +
              2 * (AllClims[eoi400][var][:, :] - AllClims[ei400][var][:, :])) / 6

    dTice = (2 * (AllClims[ei280][var][:, :] - AllClims[e280][var][:, :]) +
             (AllClims[ei400][var][:, :] - AllClims[e400][var][:, :]) +
             (AllClims[eoi280][var][:, :] - AllClims[eo280][var][:, :]) +
             2 * (AllClims[eoi400][var][:, :] - AllClims[eo400][var][:, :])) / 6

    return dT, dTCO2, dTtopo, dTice


def factorizeSharedSynergy():
    """
    Factorize an anomaly into contributions from each of the different changes to the boundary conditions
    using the all-linear and complete methodology of Lunt et al. 2020.
    
    Returns:
        1. The total change between pliocene and PI
        2. The factorized contribution from CO2
        3. The factorized contribution from topography
        4. The factorized contribution from ice
    """
    var = "atm/TREFHT"
    dT, dTCO2, dTtopo, dTice = factorize_Lunt2012()

    res = ((3 * AllClims[eoi400][var][:, :] + AllClims[ei400][var][:, :] +
            AllClims[eoi280][var][:, :] + AllClims[eo400][var][:, :]) -
           (3 * AllClims[e280][var][:, :] + AllClims[ei280][var][:, :] +
            AllClims[eo280][var][:, :] + AllClims[e400][var][:, :])) / 4

    dTCO2 = dTCO2 * (dT / res)
    dTtopo = dTtopo * (dT / res)
    dTice = dTice * (dT / res)

    return dT, dTCO2, dTtopo, dTice
